/******************************************************************************
*
*  KSGen.dsp
*
*  (c) 2001, 2002, 2003 ethan bordeaux
*  syndevkit@dspmusic.org
*
*  Karplus Strong Algorithm for Plucked Strings and Other Fun Things
*
*  author:  ethan bordeaux
*
*  assumptions:
*    M0 = 0
*    M1 = 1
*    M6 = 0
*    M7 = 1
*    I6 = address some circular random number array
*
*  a_KSGen is an array of parameters for configuring KSGen
*  p_KSGen is a frame pointer to the start of the parameters for a particular
*  KSGen call
*
*    word1:  0 - adjacent samples are subtracted
*            1 - adjacent samples are added
*    word2:  pointer into buffer of noise (must be circular!)
*    word3:  length of delay line used (determines freq)
*    word4:  determines if averaging factor is pos/neg.  range is 0xc000-0x4000
*            values near those limits lead to plucked string sounding things
*            while those closer to 0x0000 lead to percussive noise
*    word5:  averaging factor - values above 0x4000 lead to increasing feedback
*            and those less than to a short KS output.  0x4000 creates a very
*            long tone, especially for lower frequencies
*
*    the next word in a_KSGen is equivalent to word1 for the next generator
*
*    the general formula for the output of this function is:
*
*      output = (+/-)(ave factor)*(input(n) +/- input(n-1))
*
*    where the first +/- choice is controlled by word4, the second +/- choice
*    is controlled by word1, and the average factor is word5.  the inputs
*    are read from the delay line buffer and are updated every sample.
*
******************************************************************************/

.MODULE                                     modKSGen;

/*
 *  include files
 */


#include    ".\comsys\hdr\DSPdefs.h"
#include    ".\gen_fx\hdr\GenFX.h"

/*
 *  entry points
 */
 
.ENTRY      KSGen;

KSGen:
    I2 = DM(p_KSGen);
    AR = DM(I2, M1);                        /* add/sub sample indicator */
    AY0 = DM(I2, M1);                       /* get ptr into noise line */
    AY1 = DM(I2, M1);                       /* get tone freq */
    M2 = -1;
    SE = -1;    
 
    /* SR0=rand() */
    SR0 = DM(I6, M7);
    
    I3 = AY0;
    L3 = AY1;

    AX0=AR, SR=ashift SR0 (LO);             /* save add/sub var, downshift 1 */
    AY0 = DM(I2, M1);                       /* get random # threshold */
    
    /* calculate if averaging factor is pos or neg */
    AF=SR0-AY0, AR=DM(I2,M1);               /* get ave factor, rnd-thresh */
    if GE AR = -AR;
    MY0 = AR;
    
    SI = DM(I3, M1);                        /* get first sample */
    SR=ashift SI (LO), MR1=DM(I3,M2);       /* scale, get next sample, ptr-- */
    AY1=SR0, SR=ashift MR1 (LO);            /* scale, move prev sample */

    /* LP (+) or HP (-) samples? */ 
    AR = SR0 - AY1;                         /* assume subtract */
    AF = tstbit 0 of AX0;                   /* add/sub samples? */
    if EQ jump ScaleSamples;                /* if correct, skip add */
    
AddSamples:
    AR = SR0 + AY1;                         /* add scaled samples */
    
ScaleSamples:
    SR = ashift AR by 1 (LO);               /* upshift from scaledown */
    MR = SR0 * MY0 (RND);                   /* mult by averaging factor */
    DM(I3, M1) = MR1;                       /* write sample back to buffer */

WriteToOutArray:
    DM(I7, M6) = AR;
    
KSExit:
    L3 = 0;
    DM(p_KSGen) = I2;
    M2 = -4;
    modify(I2, M2);
    AR = I3;
    DM(I2, M0) = AR;                        /* inc pointer into noisebuff */

    rts;

.ENDMOD;